// Simplest example for analyzing geant4 output
// 
// .L macros/example/Example0.cc+
// Example0("root/rootfile.root")

#include <iostream>
#include <vector>
#include <math.h>

#include <TFile.h>
#include <TTree.h>
#include <TH1.h>

#include "TSimulationDataBeam.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterGlobal.hh"
#include "TSimulationParameterNEBULA.hh"
#include "TScintillatorLightOutput.hh"

//_____________________________________________________________________________
void Example0(const char* RootFileName){

  TFile *file = new TFile(RootFileName,"readonly");

  TSimulationParameterGlobal *ParaGlobal;
  file->GetObject("parameterGlobal", ParaGlobal);
  if(!ParaGlobal){
    std::cerr << " SimParaGlobal does not exist" << std::endl;
    return;
  }
  std::cout<<*ParaGlobal<<std::endl;

  TSimulationParameterNEBULA *ParaNEBULA;
  file->GetObject("parameterNEBULA", ParaNEBULA);
  if(!ParaNEBULA){
    std::cerr << " SimParaNEBULA does not exist" << std::endl;
    return;
  }
  std::cout<<*ParaNEBULA<<std::endl;

  TTree* tree;
  file->GetObject(ParaGlobal->fTreeName, tree);
  if(!tree){
    std::cerr << " cannot find tree" << std::endl;
    return;
  }

  TBranch* branchSimulationDataScintillatorArray = tree->GetBranch("neutronDetector");
  branchSimulationDataScintillatorArray->SetAddress(&gSimulationDataScintillatorArray);

  //----------------------------------------------
  TH1 *heff = new TH1I("heff","Efficiency",2,-0.5,1.5);
  TH1 *hid = new TH1I("hid","ID",144,0.5,144.5);
  TH1 *hmulti = new TH1I("hmulti","Multiplicity",10,-0.5,9.5);
  TH1 *hedepo = new TH1D("hedepo","Edepo (MeVee)",200,0,200);

  //----------------------------------------------
  //----------------------------------------------
  Double_t Edepo[144];
  for (int ieve=0;ieve<tree->GetEntries();++ieve){
    if (ieve%1000==0)
      std::cout<< "\r Event: "<<ieve
	       <<" / "<<tree->GetEntries()
	       <<" ("<<(double)ieve/tree->GetEntries()*100.<<"%)"
	       <<std::flush;

    branchSimulationDataScintillatorArray->GetEntry(ieve);

    for (int i=0;i<144;i++) Edepo[i] = 0;

    for(int i=0; i<(int)gSimulationDataScintillatorArray->size(); ++i){
      TSimulationDataScintillator DataScinti = (*gSimulationDataScintillatorArray)[i];
      std::vector<Int_t> id = DataScinti.fModuleID;
      Double_t Edepo_MeVee = TScintillatorLightOutput::MeVtoMeVee(DataScinti.fPreKineticEnergy,
								  DataScinti.fPreKineticEnergy-DataScinti.fEnergyDeposit,
								  DataScinti.fParticleName.Data(),
								  DataScinti.fPDGCode);

      Edepo[id[0]] +=  Edepo_MeVee;

    }

    Int_t IsDetected=0;
    Int_t Multi=0;
    for (int i=0;i<121;i++) 
      if (Edepo[i]>6) {
	hid->Fill(i);
	hedepo->Fill(Edepo[i]);
	Multi++;
	IsDetected=1;
      }
    heff->Fill(IsDetected);
    hmulti->Fill(Multi);
  }

  return;
}
//_____________________________________________________________________________
