#ifndef __TANALYSISEDEMOCRATICDECAY__
#define __TANALYSISEDEMOCRATICDECAY__

#include <TObject.h>
#include <TFile.h>
#include <TString.h>
#include <TH1.h>
#include <TTree.h>
#include <TBranch.h>

#include "TSimulationParameterGlobal.hh"
#include "TSimulationParameterNEBULA.hh"

class TAnalysisDataBeam;
class TAnalysisDataScintillator;

//class TAnalysisDemocraticDecay : public TObject {
class TAnalysisDemocraticDecay {

public:
  TAnalysisDemocraticDecay();
  ~TAnalysisDemocraticDecay(){}

  void ReadInputFile(TString rootfile);
  void OpenOutputFile(TString rootfile);

  void Analyze();
  void SetParameters();
  void BookHist();
  void ProcessOneEvent();
  void SimToExp();

protected:

  static const Double_t Mn = 939.565;//MeV/c2
  static const Double_t Vc = 29.9792458;// cm/ns

  TFile *fInputFile;
  TTree *fTree;

  TFile *fOutputFile;
  TString fHeader;
  TSimulationParameterGlobal *fParaGlobal;
  TSimulationParameterNEBULA *fParaNebula;

  TBranch* fBranchSimulationDataBeamArray;
  TBranch* fBranchSimulationDataScintillatorArray;

  // Parameters
  Double_t fNeutThre;
  Double_t fVetoThre;
  Double_t fTimeReso;
  Double_t fV_scinti;
  Double_t fNeutAttLen;
  Double_t fVetoAttLen;

  TH1 *fherel;

  TH1 *fhid;
  TH1 *fhmulti;
  TH1 *fhedepo;
  TH1 *fhidlayer;
  TH1 *fhidsublayer;
  TH1 *fhidsublayerseq;
  TH1 *fhidtu;
  TH1 *fhidtd;
  TH1 *fhidt;
  TH1 *fhidt_sim;
  TH1 *fhidqu;
  TH1 *fhidqd;
  TH1 *fhidq;
  TH1 *fhidx;
  TH1 *fhidy;
  TH1 *fhidz;
  TH1 *fhidx_sim;
  TH1 *fhidy_sim;
  TH1 *fhidz_sim;

  //--------------------------------
  class TAnalysisDataScintillator : public TObject {
  public:
    TAnalysisDataScintillator()
      : id(0), layer(0), sublayer(0), sublayerseq(0),
	qu(0), qd(0), tu(9999), td(9999),
	q(0), t(9999), dt(9999), t_sim(9999)
    {
      pos.SetXYZ(0,0,0);
      beta.SetXYZ(0,0,0);
      p.SetXYZ(0,0,0);
      pos_sim.SetXYZ(0,0,0);
    }
    ~TAnalysisDataScintillator(){}

    int id, layer, sublayer, sublayerseq;
    double qu,qd,tu,td;
    double q,t,dt;
    TVector3 pos;
    TVector3 beta,p;
    // before including detector responce
    double t_sim;
    double q_sim;
    TVector3 pos_sim;
    ClassDef(TAnalysisDataScintillator, 1)
  };
  //--------------------------------
  std::vector<TAnalysisDataScintillator> fvdata;


  ClassDef(TAnalysisDemocraticDecay, 1);
};



#endif
