// HOW TO USE
// root [0] .L macros/examples/GenerateTreeInput_PhaseSpaceDecay.cc+
// root [1] GenerateTreeInput_PhaseSpaceDecay()

#include "TPrimaryParticle.hh"

#include "TFile.h"
#include "TTree.h"
#include "TH1.h"
#include "TGenPhaseSpace.h"
#include "TRandom3.h"
#include "TLorentzVector.h"
#include "TVector3.h"

#include <iostream>

void GeneratePartcileTreeInput_PhaseSpaceDecay(Double_t Erel_in=1.0, Double_t E_in=250){
  // parameteres
  const int neve = 10000;

  const Int_t Abeam = 26;
  const Int_t Afrag = 24;
  const Int_t Zfrag = 8;
  const Int_t Ndecay = Abeam - Afrag + 1;

  const Int_t    Z[Ndecay]={Zfrag,0,0};
  const Int_t    A[Ndecay]={Afrag,1,1};
  const Double_t Masses[Ndecay] = {22370.8, 939.565, 939.565};

  // Init Phase Space Decay
  Double_t Mbeam = Erel_in;
  for (Int_t i=0;i<Ndecay;++i) Mbeam+=Masses[i];
  Double_t Tbeam = Abeam*E_in;
  Double_t Ebeam = Mbeam + Tbeam;
  Double_t Pbeam_abs = sqrt(Tbeam*(Tbeam+2.0*Mbeam));
  TLorentzVector Pbeam(0, 0, Pbeam_abs, Ebeam);
  TGenPhaseSpace Decay;
  Decay.SetDecay(Pbeam,Ndecay,Masses);
  //

  TFile *file = new TFile("root/input_example.root","recreate");
  TTree *tree = new TTree("tree_input","Input tree for simtrace");
  gPrimaryParticleArray = new TPrimaryParticleArray();
  tree->Branch("PrimaryParticleArray", gPrimaryParticleArray);

  // for check
  TH1* herel_gen = new TH1D("herel_gen","Erel generated",200,0,10);

  for (int i=0;i<neve;++i){
    gPrimaryParticleArray->clear();

    // position spread: uniform, +-20mm for XY, +-5mm for Z
    TVector3 pos_vec(gRandom->Uniform(-20,20),
		     gRandom->Uniform(-20,20),
		     gRandom->Uniform(-5,5)); 


    double weight = Decay.Generate();
    while (gRandom->Uniform()>weight) weight = Decay.Generate();

    TLorentzVector P[Ndecay];
    for (int i=0;i<Ndecay;++i){
      P[i] = *(Decay.GetDecay(i));
      TPrimaryParticle particle(Z[i],A[i],P[i],pos_vec); 
      gPrimaryParticleArray->push_back(particle);

    }
    
    // checking input
    TLorentzVector Ptot(0,0,0,0);
    Double_t Mtot=0;
    for (Int_t i=0;i<Ndecay;i++) {
      Ptot += P[i];
      Mtot += P[i].M();
    }
    Double_t Erel_gen = Ptot.M() - Mtot;
    herel_gen->Fill(Erel_gen);

    // Fill tree
    tree->Fill();

  }
  file->Write();
}
