#include "ActionSequentialDecayMessenger.hh"
#include "PrimaryGeneratorActionSequentialDecay.hh"
#include "ActionCommonVariable.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"

ActionSequentialDecayMessenger* ActionSequentialDecayMessenger::fActionSequentialDecayMessenger(0);

ActionSequentialDecayMessenger* ActionSequentialDecayMessenger::GetInstance()
{
  if(fActionSequentialDecayMessenger == 0){
    fActionSequentialDecayMessenger = new ActionSequentialDecayMessenger;
  }
  return fActionSequentialDecayMessenger;
}

ActionSequentialDecayMessenger::ActionSequentialDecayMessenger()
  : fPrimaryGeneratorActionSequentialDecay(0)
{
  fActionCommonVariable = ActionCommonVariable::GetInstance();

  fSeqDecayCmd = new G4UIcmdWithoutParameter("/action/SequentialDecay",this);
  fSeqDecayCmd->SetGuidance("Set Decay type to SequentialDecay");
  fSeqDecayCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  // for Decay simulation
  fActionSequentialDecayDir = new G4UIdirectory("/action/SequentialDecay/");
  fActionSequentialDecayDir->SetGuidance(" Set SequentialDecay Parameters.");

  fSeqDecayErelCmd = new G4UIcmdWithADoubleAndUnit("/action/SequentialDecay/Erel",this);
  fSeqDecayErelCmd->SetGuidance("Set Erel for Decay");
  fSeqDecayErelCmd->SetParameterName("Erel",false);
  fSeqDecayErelCmd->SetUnitCategory("Energy");
  fSeqDecayErelCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fSeqDecayErel012Cmd = new G4UIcmdWithADoubleAndUnit("/action/SequentialDecay/Erel012",this);
  fSeqDecayErel012Cmd->SetGuidance("Set Erel for 2nd Decay");
  fSeqDecayErel012Cmd->SetParameterName("Erel012",false);
  fSeqDecayErel012Cmd->SetUnitCategory("Energy");
  fSeqDecayErel012Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fSeqDecayAcoreCmd = new G4UIcmdWithAnInteger("/action/SequentialDecay/Acore",this);
  fSeqDecayAcoreCmd->SetGuidance("Define core A");
  fSeqDecayAcoreCmd->SetParameterName("Acore",false);
  fSeqDecayAcoreCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fSeqDecayZcoreCmd = new G4UIcmdWithAnInteger("/action/SequentialDecay/Zcore",this);
  fSeqDecayZcoreCmd->SetGuidance("Define core Z");
  fSeqDecayZcoreCmd->SetParameterName("Zcore",false);
  fSeqDecayZcoreCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fSeqDecayAbeamCmd = new G4UIcmdWithAnInteger("/action/SequentialDecay/Abeam",this);
  fSeqDecayAbeamCmd->SetGuidance("Define beam A");
  fSeqDecayAbeamCmd->SetParameterName("Abeam",false);
  fSeqDecayAbeamCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fSeqDecayZbeamCmd = new G4UIcmdWithAnInteger("/action/SequentialDecay/Zbeam",this);
  fSeqDecayZbeamCmd->SetGuidance("Define beam Z");
  fSeqDecayZbeamCmd->SetParameterName("Zbeam",false);
  fSeqDecayZbeamCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
}

ActionSequentialDecayMessenger::~ActionSequentialDecayMessenger()
{
  delete fSeqDecayCmd;
  delete fSeqDecayErelCmd;
  delete fSeqDecayErel012Cmd;
  delete fSeqDecayAbeamCmd;
  delete fSeqDecayZbeamCmd;
  delete fSeqDecayAcoreCmd;
  delete fSeqDecayZcoreCmd;
  delete fActionSequentialDecayDir;

  fActionSequentialDecayMessenger = 0;
}

void ActionSequentialDecayMessenger::Register(PrimaryGeneratorActionSequentialDecay* primaryGeneratorActionSequentialDecay)
{
  fPrimaryGeneratorActionSequentialDecay = primaryGeneratorActionSequentialDecay;
}

void ActionSequentialDecayMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
  if(command == fSeqDecayCmd){
    if(!fPrimaryGeneratorActionSequentialDecay->SetBeamType("SequentialDecay")){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fSeqDecayErelCmd){
    if(!fPrimaryGeneratorActionSequentialDecay->SetErel(fSeqDecayErelCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fSeqDecayErel012Cmd){
    if(!fPrimaryGeneratorActionSequentialDecay->SetErel012(fSeqDecayErel012Cmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fSeqDecayAbeamCmd){
    if(!fPrimaryGeneratorActionSequentialDecay->SetAbeam(fSeqDecayAbeamCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fSeqDecayZbeamCmd){
    if(!fPrimaryGeneratorActionSequentialDecay->SetZbeam(fSeqDecayZbeamCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fSeqDecayAcoreCmd){
    if(!fPrimaryGeneratorActionSequentialDecay->SetAcore(fSeqDecayAcoreCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fSeqDecayZcoreCmd){
    if(!fPrimaryGeneratorActionSequentialDecay->SetZcore(fSeqDecayZcoreCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }
}

