
#include "EventActionBasic.hh"
#include "ActionCommonVariable.hh"

#include "G4Event.hh"
#include "G4RunManager.hh"
#include "G4Run.hh"

#include "TDataInitializerInterface.hh"

#include "TFile.h"
#include "TTree.h"
#include "TDirectory.h"

EventActionBasic::EventActionBasic(TDataInitializerInterface* dataInitializerInterface)
  : fDataInitializerInterface(dataInitializerInterface), fTimeLast(0), fTimeStart(0)
{
  fActionCommonVariable = ActionCommonVariable::GetInstance();
}

EventActionBasic::~EventActionBasic()
{;}

void EventActionBasic::BeginOfEventAction(const G4Event* anEvent)
{
  if(anEvent->GetEventID()==0){
    time(&fTimeStart);
    fTimeLast = time(0);
  }

  G4RunManager *RunManager = G4RunManager::GetRunManager();
  const G4Run *Run = RunManager->GetCurrentRun();

  time_t timeNow = time(0);
  if(timeNow-fTimeLast>=1){
    G4cout << "\r Event:" << anEvent->GetEventID()+1 << " ("
	   << (double)(anEvent->GetEventID()+1)/Run->GetNumberOfEventToBeProcessed()*100.  
	   << "%), "
	   << (anEvent->GetEventID()+1)/(timeNow-fTimeStart) << " events/sec, "
	   << timeNow-fTimeStart << " sec elapsed." << std::flush;
    fTimeLast = timeNow;
  }else if(timeNow-fTimeLast<0){
    fTimeLast = timeNow;
  }

  fDataInitializerInterface->ClearBuffer();
}

void EventActionBasic::EndOfEventAction(const G4Event* anEvent)
{
  if((anEvent->GetEventID()+1) % 10000 == 0){
    gFile->Write();
  }

  if(fActionCommonVariable->GetUseTree()){
    TTree* tree;
    gDirectory->GetObject(fActionCommonVariable->GetTreeName(), tree);
    tree->Fill();
  }
}  
