#ifndef DETECTORCONSTRUCTIONMONA_HH
#define DETECTORCONSTRUCTIONMONA_HH

#include "G4VUserDetectorConstruction.hh"
#include <TString.h>
#include <vector>


class G4LogicalVolume;
class G4Material;
class G4VSensitiveDetector;
class DetectorConstruction;
class DetectorConstructionMoNAMessenger;
class TargetConstruction;
class NeutronWindowConstruction;
class DipoleConstruction;
class NeutronDetectorConstruction;
class TSimulationParameterMoNA;


class DetectorConstructionMoNA : public G4VUserDetectorConstruction
{
public:
  DetectorConstructionMoNA();
  virtual ~DetectorConstructionMoNA();

  G4VPhysicalVolume* Construct();
  G4LogicalVolume* ConstructSub();

  void UpdateGeometry();
  void ReadParameterNeutronDetector(const char* PrmFileName);
  void ReadParameterMoNA(const char* PrmFileName);
  void CSVToVector(const char* PrmFileName, std::vector<std::vector<TString> >* PrmArray);

protected:
  DetectorConstructionMoNAMessenger* fDetectorConstructionMoNAMessenger;

  TargetConstruction* fTargetConstruction;
  NeutronWindowConstruction* fNeutronWindowConstruction;
  DipoleConstruction* fDipoleConstruction;
  NeutronDetectorConstruction* fNeutronDetectorConstruction;

  G4LogicalVolume* fLogicWorld;
  G4LogicalVolume* fLogicTarget;
  G4LogicalVolume* fLogicNeutronWindow;
  G4LogicalVolume* fLogicDipole;
  G4LogicalVolume* fLogicMoNA;
  G4LogicalVolume* fLogicNeut;
  G4LogicalVolume* fLogicVeto;
  G4LogicalVolume* fLogicAir;

  G4Material* fWorldMaterial;
  G4Material* fAirMaterial;
  G4Material* fMoNAMaterial;// AIR/Galactic for Mother volume of detectors
  G4Material* fNeutMaterial;

  G4VSensitiveDetector* fScintillatorSD;

};

#endif

