#ifndef DETECTORCONSTRUCTIONNEBULA_HH
#define DETECTORCONSTRUCTIONNEBULA_HH

#include "G4VUserDetectorConstruction.hh"
#include <TString.h>
#include <vector>


class G4LogicalVolume;
class G4Material;
class G4VSensitiveDetector;
class DetectorConstruction;
class DetectorConstructionNEBULAMessenger;
class TargetConstruction;
class NeutronWindowConstruction;
class DipoleConstruction;
class NeutronDetectorConstruction;
class TSimulationParameterNEBULA;


class DetectorConstructionNEBULA : public G4VUserDetectorConstruction
{
public:
  DetectorConstructionNEBULA();
  virtual ~DetectorConstructionNEBULA();

  G4VPhysicalVolume* Construct();
  G4LogicalVolume* ConstructSub();

  void UpdateGeometry();
  void ReadParameterNeutronDetector(const char* PrmFileName);
  void ReadParameterNEBULA(const char* PrmFileName);
  void CSVToVector(const char* PrmFileName, std::vector<std::vector<TString> >* PrmArray);

protected:
  DetectorConstructionNEBULAMessenger* fDetectorConstructionNEBULAMessenger;

  TargetConstruction* fTargetConstruction;
  NeutronWindowConstruction* fNeutronWindowConstruction;
  DipoleConstruction* fDipoleConstruction;
  NeutronDetectorConstruction* fNeutronDetectorConstruction;

  G4LogicalVolume* fLogicWorld;
  G4LogicalVolume* fLogicTarget;
  G4LogicalVolume* fLogicNeutronWindow;
  G4LogicalVolume* fLogicDipole;
  G4LogicalVolume* fLogicNEBULA;
  G4LogicalVolume* fLogicNeut;
  G4LogicalVolume* fLogicVeto;
  G4LogicalVolume* fLogicAir;

  G4Material* fWorldMaterial;
  G4Material* fAirMaterial;
  G4Material* fNEBULAMaterial;// AIR/Galactic for Mother volume of detectors
  G4Material* fNeutMaterial;

  G4VSensitiveDetector* fScintillatorSD;

};

#endif

