
#include "DetectorConstructionMoNAMessenger.hh"
#include "DetectorConstructionMoNA.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "G4UIcmdWithAString.hh"

DetectorConstructionMoNAMessenger::DetectorConstructionMoNAMessenger(DetectorConstructionMoNA* Det)
 : fDetectorConstructionMoNA(Det)
{
  fDetectorDirectory = new G4UIdirectory("/detector/");
  fDetectorDirectory->SetGuidance("Modification Commands for Detector");
  fDetectorMoNADirectory = new G4UIdirectory("/detector/MoNA/");
  fDetectorMoNADirectory->SetGuidance("Modification Commands for MoNA Detector");

  fUpdateCmd = new G4UIcmdWithoutParameter("/detector/update",this);
  fUpdateCmd->SetGuidance("Update geometry.");
  fUpdateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  fUpdateCmd->SetGuidance("if you changed geometrical value(s)");
  fUpdateCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

//  fParaNeutDetecCmd = new G4UIcmdWithAString("/detector/ParaNeutronDetectors",this);
//  fParaNeutDetecCmd->SetGuidance("Parameter file name for Neutron Detectors.");
//  fParaNeutDetecCmd->SetParameterName("ParaNeutDetectros",false);
  fParaNeutDetecCmd = new G4UIcmdWithAString("/detector/MoNA/ParaNeutronDetectors",this);
  fParaNeutDetecCmd->SetGuidance("Parameter file name for Neutron Detectors.");
  fParaNeutDetecCmd->SetParameterName("ParaNeutDetectros",false);

  fParaMoNACmd = new G4UIcmdWithAString("/detector/ParaMoNA",this);
  fParaMoNACmd->SetGuidance("Parameter file name for MoNA.");
  fParaMoNACmd->SetParameterName("ParaMoNA",false);
}

DetectorConstructionMoNAMessenger::~DetectorConstructionMoNAMessenger()
{
  delete fDetectorDirectory;
  delete fUpdateCmd;
  delete fParaNeutDetecCmd;
  delete fParaMoNACmd;
}

void DetectorConstructionMoNAMessenger::SetNewValue(G4UIcommand* command, G4String  newValue )
{
  if(command == fUpdateCmd) fDetectorConstructionMoNA->UpdateGeometry();

  if(command == fParaNeutDetecCmd) 
    fDetectorConstructionMoNA->ReadParameterNeutronDetector(newValue);

  if(command == fParaMoNACmd) 
    fDetectorConstructionMoNA->ReadParameterMoNA(newValue);
}


