
#include "DetectorConstructionNEBULA.hh"
#include "DetectorConstructionNEBULAMessenger.hh"

#include "TargetConstruction.hh"
#include "NeutronWindowConstruction.hh"
#include "DipoleConstruction.hh"

#include "TSimulationParameterNeutronDetector.hh"
#include "TSimulationParameterNEBULA.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

#include "G4SDManager.hh"
#include "G4VSensitiveDetector.hh"
#include "ScintillatorSD.hh"

#include <string>
#include <iostream>
#include <sstream>
#include <algorithm>
//______________________________________________________________________________

DetectorConstructionNEBULA::DetectorConstructionNEBULA()
  : fLogicWorld(0), fLogicTarget(0), fLogicNeutronWindow(0), fLogicDipole(0), fLogicNEBULA(0), fLogicAir(0),
    fScintillatorSD(0)
{
  fDetectorConstructionNEBULAMessenger = new DetectorConstructionNEBULAMessenger(this);

  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
  fAirMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  //  fAirMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
  fNeutMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");
  fNEBULAMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");

  fTargetConstruction = new TargetConstruction;
  fNeutronWindowConstruction = new NeutronWindowConstruction;
  fDipoleConstruction = new DipoleConstruction;

  if (gSimulationParameterNEBULA==0)
    gSimulationParameterNEBULA = new TSimulationParameterNEBULA();
}

//______________________________________________________________________________
DetectorConstructionNEBULA::~DetectorConstructionNEBULA()
{
  delete fTargetConstruction;
  delete fNeutronWindowConstruction;
  delete fDipoleConstruction;
}
//______________________________________________________________________________
G4VPhysicalVolume* DetectorConstructionNEBULA::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  fLogicNEBULA = ConstructSub();
//  fLogicTarget = fTargetConstruction->ConstructSub();
//  fLogicNeutronWindow = fNeutronWindowConstruction->ConstructSub();
//  fLogicDipole = fDipoleConstruction->ConstructSub();

  // -------------------------------------------------

  // World LV
  G4ThreeVector worldSize(1.1*2.*fabs(0.5*gSimulationParameterNEBULA->fSize.x()+gSimulationParameterNEBULA->fPosition.x())*cm,
			  1.1*2.*fabs(0.5*gSimulationParameterNEBULA->fSize.y()+gSimulationParameterNEBULA->fPosition.y())*cm,
			  1.1*2.*fabs(0.5*gSimulationParameterNEBULA->fSize.z()+gSimulationParameterNEBULA->fPosition.z())*cm );

  G4Box* solidWorld = new G4Box("World",
				0.5*worldSize.x(),
				0.5*worldSize.y(),
				0.5*worldSize.z());
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");

  // World PV
  G4ThreeVector worldPos(0,0,0);
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);
  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);
  //-------------------------------------------------------
  G4ThreeVector nebulaPos(gSimulationParameterNEBULA->fPosition.x()*cm,
			  gSimulationParameterNEBULA->fPosition.y()*cm,
			  gSimulationParameterNEBULA->fPosition.z()*cm);
  // NEBULA PV
  G4RotationMatrix RotMat = G4RotationMatrix();
  RotMat.rotateX(gSimulationParameterNEBULA->fRotAngle.x()*deg);
  RotMat.rotateY(gSimulationParameterNEBULA->fRotAngle.y()*deg);
  RotMat.rotateZ(gSimulationParameterNEBULA->fRotAngle.z()*deg);
 
//  new G4PVPlacement(0,
//		    nebulaPos,
  new G4PVPlacement(G4Transform3D(RotMat, nebulaPos),
		    fLogicNEBULA,
		    "NEBULA",
		    fLogicWorld,
		    false,
		    0);
//  fLogicNEBULA->SetVisAttributes(G4VisAttributes::Invisible);
  
//  // target PV
//  new G4PVPlacement(0,
//		    targetPos,
//		    fLogicTarget,
//		    "Target",
//		    fLogicWorld,
//		    false,
//		    0);
//
//  // neutron window PV
//  new G4PVPlacement(0,
//		    neutronWindowPos,
//		    fLogicNeutronWindow,
//		    "NeutronWindow",
//		    fLogicWorld,
//		    false,
//		    0);
//
////  // dipole PV
////  new G4PVPlacement(0,
////		    dipolePos,
////		    fLogicDipole,
////		    "Dipole",
////		    fLogicWorld,
////		    false,
////		    0);
//
//  // air PV
//  new G4PVPlacement(0,
//		    airPos,
//		    fLogicAir,
//		    "Air",
//		    fLogicWorld,
//		    false,
//		    0);

//  // Position
//  G4ThreeVector targetPos(0, 0, 0);
//  G4ThreeVector neutronWindowPos(0, 0, 6*m);
//  G4ThreeVector dipolePos(0, 0, neutronWindowPos.z()-fDipoleConstruction->GetDipoleSize().z()/2);
//  // Construct Air between neutron window and NEBULA LV
//  G4ThreeVector airSize(4*m,
//			3*m,
//			(nebulaPos.z()-fNeutronDetectorConstruction->GetNeutronDetectorSize().z()/2)
//			-(neutronWindowPos.z()+fNeutronWindowConstruction->GetNeutronWindowSize().z()/2));
//  G4ThreeVector airPos(0, 0,
//		       neutronWindowPos.z()+fNeutronWindowConstruction->GetNeutronWindowSize().z()/2
//		       +airSize.z()/2);
//  G4Box* solidAir = new G4Box("Air",
//			      airSize.x()/2,
//			      airSize.y()/2,
//			      airSize.z()/2);
//  fLogicAir = new G4LogicalVolume(solidAir,fAirMaterial,"Air");
//
//  //  G4VisAttributes* airColor = new G4VisAttributes(G4Colour::Gray());
////  fLogicAir->SetVisAttributes(airColor);


//  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}
//______________________________________________________________________________
G4LogicalVolume* DetectorConstructionNEBULA::ConstructSub()
{

  G4cout<< "ConstructSub : "<<*gSimulationParameterNEBULA <<G4endl;
  std::map<int, TSimulationParameterNeutronDetector*>ParaMap 
    = gSimulationParameterNEBULA->fParaNeutDetecMap;

  G4Box* solidNEBULA = new G4Box("NEBULA",
				 0.5*gSimulationParameterNEBULA->fSize.x()*cm,
				 0.5*gSimulationParameterNEBULA->fSize.y()*cm,
				 0.5*gSimulationParameterNEBULA->fSize.z()*cm);
  
  fLogicNEBULA = new G4LogicalVolume(solidNEBULA,fNEBULAMaterial,"NeutronDetector");
  
  // placement of each module
  G4Box* solidNeut = new G4Box("NeutronDetector",
			       gSimulationParameterNEBULA->fNeutSize.x()*0.5*cm,
			       gSimulationParameterNEBULA->fNeutSize.y()*0.5*cm,
			       gSimulationParameterNEBULA->fNeutSize.z()*0.5*cm);
  fLogicNeut   = new G4LogicalVolume(solidNeut,fNeutMaterial,"NeutronDetector");

  if (gSimulationParameterNEBULA->fVetoNum>0) {
    G4Box* solidVeto = new G4Box("VetoDetector",
				 gSimulationParameterNEBULA->fVetoSize.x()*0.5*cm,
				 gSimulationParameterNEBULA->fVetoSize.y()*0.5*cm,
				 gSimulationParameterNEBULA->fVetoSize.z()*0.5*cm);
    fLogicVeto   = new G4LogicalVolume(solidVeto,fNeutMaterial,"VetoDetector");
  }


  std::map<int,TSimulationParameterNeutronDetector*>::iterator it = ParaMap.begin();
  while (it != ParaMap.end()){
    TSimulationParameterNeutronDetector *para = (*it).second;

    G4ThreeVector detPos(para->fPosition.x()*cm,
			 para->fPosition.y()*cm, 
			 para->fPosition.z()*cm);

    // Rotation
    G4RotationMatrix RotMat = G4RotationMatrix();
    RotMat.rotateX(para->fRotAngle.x()*deg);
    RotMat.rotateY(para->fRotAngle.y()*deg);
    RotMat.rotateZ(para->fRotAngle.z()*deg);

    G4LogicalVolume *lv=0;
    if      (para->fDetecType=="Neut") lv = fLogicNeut;
    else if (para->fDetecType=="Veto") lv = fLogicVeto;
 
    new G4PVPlacement(G4Transform3D(RotMat, detPos),
		      lv,
		      (para->fDetecType).Data(),
		      fLogicNEBULA,
		      false,// Many <- should be checked
		      para->fID);// Copy num <- should be checked
    ++it;
  }

  // Sensitive Detector
  if (fScintillatorSD==0){
    G4SDManager *SDman = G4SDManager::GetSDMpointer();
    fScintillatorSD = new ScintillatorSD("/ScintillatorNeutronDetector");
    SDman->AddNewDetector(fScintillatorSD);
  }

  fLogicNeut->SetSensitiveDetector(fScintillatorSD);
  fLogicNeut->SetVisAttributes(new G4VisAttributes(G4Colour::Yellow()));

  if (gSimulationParameterNEBULA->fVetoNum>0) {
    fLogicVeto->SetSensitiveDetector(fScintillatorSD);
    fLogicVeto->SetVisAttributes(new G4VisAttributes(G4Colour::Cyan()));
  }    
  
  return fLogicNEBULA;
}
//______________________________________________________________________________
void DetectorConstructionNEBULA::UpdateGeometry()
{
  G4RunManager::GetRunManager()->DefineWorldVolume(Construct());
}
//______________________________________________________________________________
void DetectorConstructionNEBULA::ReadParameterNeutronDetector(const char* PrmFileName)
{
  std::vector<std::vector<TString> > PrmArray;
  CSVToVector(PrmFileName, &PrmArray);
  if (PrmArray.size()==0) {
    G4cout<< "\x1b[33m" // yellow colored text
	  << __FILE__ << ": No Parameter is Loaded from "<< PrmFileName 
	  << "\x1b[0m" // reset color
	  <<G4endl;
    return;
  }

  std::map<int,TSimulationParameterNeutronDetector*>* map = &(gSimulationParameterNEBULA->fParaNeutDetecMap);

  std::vector<TString> str_line0 = PrmArray[0];// 1st line = list of Name of PRM
  for (int iline=1;iline<(int)PrmArray.size();++iline){
    TSimulationParameterNeutronDetector *para = new TSimulationParameterNeutronDetector();
    std::vector<TString> str_line = PrmArray[iline];
    int id=0;
    
    for (int i=0;i<(int)str_line.size();++i){
      if      ("ID"       ==str_line0[i]) {
	id = str_line[i].Atoi();
	(*para).fID     = id;
      }
      else if ("Layer"    ==str_line0[i]) (*para).fLayer  = str_line[i].Atoi();
      else if ("SubLayer" ==str_line0[i]) (*para).fSubLayer = str_line[i].Atoi();

      else if ("DetecType"==str_line0[i]) (*para).fDetecType = str_line[i];
	
      else if ("PositionX"==str_line0[i]) (*para).fPosition.SetX(str_line[i].Atof());
      else if ("PositionY"==str_line0[i]) (*para).fPosition.SetY(str_line[i].Atof());
      else if ("PositionZ"==str_line0[i]) (*para).fPosition.SetZ(str_line[i].Atof());
	
      else if ("RotAngleX"==str_line0[i]) (*para).fRotAngle.SetX(str_line[i].Atof());
      else if ("RotAngleY"==str_line0[i]) (*para).fRotAngle.SetY(str_line[i].Atof());
      else if ("RotAngleZ"==str_line0[i]) (*para).fRotAngle.SetZ(str_line[i].Atof());
	
    }

    if (para->fLayer!=0 && para->fSubLayer!=0 )
      para->fSubLayerSeq = (para->fLayer-1)*2 + para->fSubLayer;

    map->insert(std::pair<int,TSimulationParameterNeutronDetector*>(id,para));
  }
  //-------------------------------------------------
  // Set Parameteres like the number of detectors etc.
  if (! gSimulationParameterNEBULA->fIsLoaded ){
    G4cout<< "\x1b[31m" // red colored text
	  << "Error @  "<<__FILE__ << G4endl
	  <<"TSimulationParameterNEBULA must be Loaded before reading "
	  << PrmFileName
	  << "\x1b[0m" // reset color
	  <<G4endl;
    return;
  }

  std::map<int,TSimulationParameterNeutronDetector*>::iterator it = map->begin();
  while (it != map->end()){
    TSimulationParameterNeutronDetector *para = (*it).second;
    TVector3 Size;// Detector size
    if        (para->fDetecType=="Neut"){
      gSimulationParameterNEBULA->fNeutNum++;
      if (para->fLayer==1) gSimulationParameterNEBULA->fNeutNumLayer++;
      if (para->fLayer==1 && para->fSubLayer==1) gSimulationParameterNEBULA->fNeutNumSubLayer++;
      Size = gSimulationParameterNEBULA->fNeutSize;

    }else if  (para->fDetecType=="Veto"){
      gSimulationParameterNEBULA->fVetoNum++;
      if (para->fLayer==1) gSimulationParameterNEBULA->fVetoNumLayer++;
      if (para->fLayer==1 && para->fSubLayer==1) gSimulationParameterNEBULA->fVetoNumSubLayer++;
      Size = gSimulationParameterNEBULA->fVetoSize;
    }

    gSimulationParameterNEBULA->fLayerNum = std::max(para->fLayer,gSimulationParameterNEBULA->fLayerNum);
    gSimulationParameterNEBULA->fSubLayerNum  = std::max(para->fSubLayerSeq,gSimulationParameterNEBULA->fSubLayerNum);

    Double_t LeftEdge   = para->fPosition.x() - 0.5*Size.x();
    Double_t RightEdge  = para->fPosition.x() + 0.5*Size.x();
    Double_t BottomEdge = para->fPosition.y() - 0.5*Size.y();
    Double_t TopEdge    = para->fPosition.y() + 0.5*Size.y();
    Double_t FrontEdge  = para->fPosition.z() - 0.5*Size.z();
    Double_t BackEdge   = para->fPosition.z() + 0.5*Size.z();

    Double_t MaxX = std::max(fabs(LeftEdge),  fabs(RightEdge));
    Double_t MaxY = std::max(fabs(BottomEdge),fabs(TopEdge));
    Double_t MaxZ = std::max(fabs(FrontEdge), fabs(BackEdge));

    gSimulationParameterNEBULA->fSize.SetX( std::max(2.0*MaxX, gSimulationParameterNEBULA->fSize.x()) );
    gSimulationParameterNEBULA->fSize.SetY( std::max(2.0*MaxY, gSimulationParameterNEBULA->fSize.y()) );
    gSimulationParameterNEBULA->fSize.SetZ( std::max(2.0*MaxZ, gSimulationParameterNEBULA->fSize.z()) );
    // Rotation is not supported...

    it++;
  }

  G4cout<< "ReadParamNeutronDetector : "<<*gSimulationParameterNEBULA <<G4endl;
  G4RunManager::GetRunManager()->GeometryHasBeenModified();
}
//______________________________________________________________________________
void DetectorConstructionNEBULA::ReadParameterNEBULA(const char* PrmFileName)
{
  std::vector<std::vector<TString> > PrmArray;
  CSVToVector(PrmFileName, &PrmArray);
  if (PrmArray.size()==0) {
    G4cout<< "\x1b[33m" // yellow colored text
	  << __FILE__ << ": No Parameter is Loaded from "<< PrmFileName 
	  << "\x1b[0m" // red colored text
	  <<G4endl;
    return;
  }

  for (int iline=0;iline<(int)PrmArray.size();++iline){
      std::vector<TString> str_line = PrmArray[iline];
      if      ("Position"==str_line[0]) (*gSimulationParameterNEBULA).fPosition.SetXYZ(str_line[1].Atof(),
										       str_line[2].Atof(),
										       str_line[3].Atof());

      if      ("NeutSize"==str_line[0]) (*gSimulationParameterNEBULA).fNeutSize.SetXYZ(str_line[1].Atof(),
										       str_line[2].Atof(),
										       str_line[3].Atof());

      if      ("VetoSize"==str_line[0]) (*gSimulationParameterNEBULA).fVetoSize.SetXYZ(str_line[1].Atof(),
										       str_line[2].Atof(),
										       str_line[3].Atof());

      else if ("RotAngle"==str_line[0]) (*gSimulationParameterNEBULA).fRotAngle.SetXYZ(str_line[1].Atof(),
										       str_line[2].Atof(),
										       str_line[3].Atof());
  }
//  G4cout<< "ReadParamNEBULA : "<<*gSimulationParameterNEBULA <<G4endl;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
    gSimulationParameterNEBULA->fIsLoaded = true;
}
//______________________________________________________________________________
void DetectorConstructionNEBULA::CSVToVector
(const char* PrmFileName, std::vector<std::vector<TString> >* PrmArray)
{
  std::ifstream ifs(PrmFileName);
  if (ifs.fail()) {
    G4cout << "\x1b[31m" // red colored text
	   <<__FILE__<<": File open error \""<<PrmFileName<<"\""
	   << "\x1b[0m" // reset color
	   <<G4endl;
    return;
  }

  std::string line;
  int iline=0;
  while(getline(ifs,line,'\n')){

    std::vector<TString>* str_line = new std::vector<TString>;

    /* Skip space line */
    TString tsline=line;
    tsline=tsline.Strip(TString::kBoth);
    if (tsline.IsNull()) {iline++;continue;}

    // skip comment line
    if (tsline.First("//")==0) {iline++;continue;}

    //--------------------------------------
    /* Scan one line */
    std::istringstream ss(line);
    std::string s;
    while (getline(ss,s,',')){
      TString ts=s;
      ts=ts.Strip(TString::kBoth);
      if (ts.IsNull()) continue;// skip space column
      str_line->push_back(ts);
    }

    PrmArray->push_back(*str_line);
    ++iline;//increment of line number
  }
  ifs.close();
}
//______________________________________________________________________________

