
#include "DetectorConstructionNEBULAMessenger.hh"
#include "DetectorConstructionNEBULA.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "G4UIcmdWithAString.hh"

DetectorConstructionNEBULAMessenger::DetectorConstructionNEBULAMessenger(DetectorConstructionNEBULA* Det)
 : fDetectorConstructionNEBULA(Det)
{
  fDetectorDirectory = new G4UIdirectory("/detector/");
  fDetectorDirectory->SetGuidance("Modification Commands for Detector");

  fUpdateCmd = new G4UIcmdWithoutParameter("/detector/update",this);
  fUpdateCmd->SetGuidance("Update geometry.");
  fUpdateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  fUpdateCmd->SetGuidance("if you changed geometrical value(s)");
  fUpdateCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fParaNeutDetecCmd = new G4UIcmdWithAString("/detector/ParaNeutronDetectors",this);
  fParaNeutDetecCmd->SetGuidance("Parameter file name for Neutron Detectors.");
  fParaNeutDetecCmd->SetParameterName("ParaNeutDetectros",false);

  fParaNEBULACmd = new G4UIcmdWithAString("/detector/ParaNEBULA",this);
  fParaNEBULACmd->SetGuidance("Parameter file name for NEBULA.");
  fParaNEBULACmd->SetParameterName("ParaNEBULA",false);
}

DetectorConstructionNEBULAMessenger::~DetectorConstructionNEBULAMessenger()
{
  delete fDetectorDirectory;
  delete fUpdateCmd;
  delete fParaNeutDetecCmd;
  delete fParaNEBULACmd;
}

void DetectorConstructionNEBULAMessenger::SetNewValue(G4UIcommand* command, G4String  newValue )
{
  if(command == fUpdateCmd) fDetectorConstructionNEBULA->UpdateGeometry();

  if(command == fParaNeutDetecCmd) 
    fDetectorConstructionNEBULA->ReadParameterNeutronDetector(newValue);

  if(command == fParaNEBULACmd) 
    fDetectorConstructionNEBULA->ReadParameterNEBULA(newValue);
}


