
#include "DetectorConstructionNeuLANDMessenger.hh"
#include "DetectorConstructionNeuLAND.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "G4UIcmdWithAString.hh"

DetectorConstructionNeuLANDMessenger::DetectorConstructionNeuLANDMessenger(DetectorConstructionNeuLAND* Det)
 : fDetectorConstructionNeuLAND(Det)
{
  fDetectorDirectory = new G4UIdirectory("/detector/");
  fDetectorDirectory->SetGuidance("Modification Commands for Detector");
  fDetectorNeuLANDDirectory = new G4UIdirectory("/detector/NeuLAND/");
  fDetectorNeuLANDDirectory->SetGuidance("Modification Commands for NeuLAND Detector");

  fUpdateCmd = new G4UIcmdWithoutParameter("/detector/update",this);
  fUpdateCmd->SetGuidance("Update geometry.");
  fUpdateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  fUpdateCmd->SetGuidance("if you changed geometrical value(s)");
  fUpdateCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

//  fParaNeutDetecCmd = new G4UIcmdWithAString("/detector/ParaNeutronDetectors",this);
//  fParaNeutDetecCmd->SetGuidance("Parameter file name for Neutron Detectors.");
//  fParaNeutDetecCmd->SetParameterName("ParaNeutDetectros",false);
  fParaNeutDetecCmd = new G4UIcmdWithAString("/detector/NeuLAND/ParaNeutronDetectors",this);
  fParaNeutDetecCmd->SetGuidance("Parameter file name for Neutron Detectors.");
  fParaNeutDetecCmd->SetParameterName("ParaNeutDetectros",false);

  fParaNeuLANDCmd = new G4UIcmdWithAString("/detector/ParaNeuLAND",this);
  fParaNeuLANDCmd->SetGuidance("Parameter file name for NeuLAND.");
  fParaNeuLANDCmd->SetParameterName("ParaNeuLAND",false);
}

DetectorConstructionNeuLANDMessenger::~DetectorConstructionNeuLANDMessenger()
{
  delete fDetectorDirectory;
  delete fUpdateCmd;
  delete fParaNeutDetecCmd;
  delete fParaNeuLANDCmd;
}

void DetectorConstructionNeuLANDMessenger::SetNewValue(G4UIcommand* command, G4String  newValue )
{
  if(command == fUpdateCmd) fDetectorConstructionNeuLAND->UpdateGeometry();

  if(command == fParaNeutDetecCmd) 
    fDetectorConstructionNeuLAND->ReadParameterNeutronDetector(newValue);

  if(command == fParaNeuLANDCmd) 
    fDetectorConstructionNeuLAND->ReadParameterNeuLAND(newValue);
}


