
#include "TDataInitializerNeutronDetector.hh"

#include "TFile.h"
#include "TTree.h"

#include <iostream>

#include "TSimulationDataBeam.hh"
#include "TSimulationParameterGlobal.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterNeutronDetector.hh"
#include "TSimulationParameterNEBULA.hh"
#include "TSimulationParameterNeuLAND.hh"
#include "TSimulationParameterMoNA.hh"

ClassImp(TDataInitializerNeutronDetector)

TDataInitializerNeutronDetector::TDataInitializerNeutronDetector()
  : TDataInitializerInterface()
{;}

TDataInitializerNeutronDetector::~TDataInitializerNeutronDetector()
{;}

int TDataInitializerNeutronDetector::Initialize()
{
  delete gSimulationDataBeamArray; gSimulationDataBeamArray = 0;
  delete gSimulationDataScintillatorArray; gSimulationDataScintillatorArray = 0;
  gSimulationDataBeamArray = new TSimulationDataBeamArray;
  gSimulationDataScintillatorArray = new TSimulationDataScintillatorArray;

  return 1;
}

int TDataInitializerNeutronDetector::CreateBranch(TTree* tree)
{
  tree->Branch("beam", gSimulationDataBeamArray);
  tree->Branch("neutronDetector", gSimulationDataScintillatorArray);

  return 1;
}

int TDataInitializerNeutronDetector::AddParameters(TFile* file)
{
  file->Add(gSimulationParameterGlobal);
//  file->Add(gSimulationParameterNeutronDetector);
  if (gSimulationParameterNEBULA!=0) file->Add(gSimulationParameterNEBULA);
  if (gSimulationParameterNeuLAND!=0) file->Add(gSimulationParameterNeuLAND);
  if (gSimulationParameterMoNA!=0) file->Add(gSimulationParameterMoNA);

  return 1;
}

int TDataInitializerNeutronDetector::RemoveParameters(TFile* file)
{
  file->Remove(gSimulationParameterGlobal);
//  file->Remove(gSimulationParameterNeutronDetector);
  if (gSimulationParameterNEBULA!=0) file->Remove(gSimulationParameterNEBULA);
  if (gSimulationParameterNeuLAND!=0) file->Remove(gSimulationParameterNeuLAND);
  if (gSimulationParameterMoNA!=0) file->Remove(gSimulationParameterMoNA);

  return 1;
}

int TDataInitializerNeutronDetector::PrintParameters(TFile* file)
{
  std::cout << "\n parameters of TDataInitializerNeutronDetector:" << std::endl;

  TSimulationParameterGlobal* simParaGlobal;
  TSimulationParameterNeutronDetector* simParaNeut;
  TSimulationParameterNEBULA* simParaNEBULA;
  TSimulationParameterNeuLAND* simParaNeuLAND;
  TSimulationParameterNeuLAND* simParaMoNA;

  file->GetObject("parameterGlobal", simParaGlobal);
//  file->GetObject("parameterNeutronDetector", simParaNeut);
  file->GetObject("parameterNEBULA", simParaNEBULA);
  file->GetObject("parameterNeuLAND", simParaNeuLAND);
  file->GetObject("parameterMoNA", simParaMoNA);
  if(!simParaGlobal){
    std::cerr << " SimParaGlobal does not exist" << std::endl;
    return 0;
  }else if(!simParaNEBULA){
    std::cerr << " SimParaNEBULA does not exist" << std::endl;
    return 0;
  }else if(!simParaNeuLAND){
    std::cerr << " SimParaNeuLAND does not exist" << std::endl;
    return 0;
  }else if(!simParaMoNA){
    std::cerr << " SimParaMoNA does not exist" << std::endl;
    return 0;
  }
  std::cout << " " << *simParaGlobal << "\n"
	    << " " << *simParaNEBULA << "\n"
	    << " " << *simParaNeuLAND << "\n"
	    << " " << *simParaMoNA << "\n"
//	    << " " << *simParaNeut << "\n"
    	    << std::endl;

  return 1;
}

int TDataInitializerNeutronDetector::ClearBuffer()
{
  gSimulationDataScintillatorArray->clear();

  return 1;
}
